<?php

namespace Drupal\webform_pipedrive\Plugin\WebformHandler;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
/**
 * Webform Pipedrive handler.
 *
 * @WebformHandler(
 *   id = "webform_pipedrive",
 *   label = @Translation("WebformPipedrive"),
 *   category = @Translation("WebformPipedrive"),
 *   description = @Translation("Send the values to the api on form submission."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_OPTIONAL,
 *   tokens = TRUE,
 * )
 */
class WebformPipedrive extends WebformHandlerBase {
  use StringTranslationTrait;
  /**
   * The entity Field Manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The language Manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected ClientInterface $httpClient;

   /**
   * Logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;


  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $plugin = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $plugin->entityFieldManager = $container->get('entity_field.manager');
    $plugin->languageManager = $container->get('language_manager');
    $plugin->httpClient = $container->get('http_client');
    $plugin->logger = $container->get('logger.channel.default');
    return $plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'enable_mapping' => FALSE,
      'enable_orgdeals' => FALSE,
      'enable_deals' => FALSE,
      'user_field_mapping' => [],
      'user_field_mapping_org' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $mapping_options = $this->getMappingOptions();

    // Checkbox to enable or disable mapping.
    $form['mapping']['enable_mapping'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Is Organisation'),
      '#description' => $this->t('Check this box to enable mapping of webform elements to Organisation fields.'),
      '#default_value' => $this->configuration['enable_mapping'],
      '#parents' => ['settings', 'enable_mapping'],
    ];

    // Container for checkboxes related to Organisation Deals API.
    $form['mapping']['user_field_mapping_org'] = [
      '#type' => 'webform_mapping',
      '#description' => $this->t('Map webform element values to organisation fields.'),
      '#required' => FALSE,
      '#source' => $mapping_options['source'],
      '#destination' => $mapping_options['orgdestination'],
      '#default_value' => $this->configuration['user_field_mapping_org'],
      '#source__title' => $this->t('Organisation Webform element'),
      '#destination__type' => 'select',
      '#destination__title' => $this->t('Organisation field destination'),
      '#destination__description' => NULL,
      '#parents' => ['settings', 'user_field_mapping_org'],
      '#states' => [
        'visible' => [
          ':input[name="settings[enable_mapping]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Container for checkboxes related to Organisation Deals API.
    $form['mapping']['user_field_mapping_org']['checkbox_container'] = [
      '#type' => 'container',
      '#weight' => 10,
    ];

    // Checkbox to enable or disable Organisation Deals API.
    $form['mapping']['user_field_mapping_org']['checkbox_container']['enable_orgdeals'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Organisation Deals API'),
      '#description' => $this->t('Check this box to enable Deals API for Organisation fields.'),
      '#default_value' => $this->configuration['enable_orgdeals'],
      '#parents' => ['settings', 'enable_orgdeals'],
    ];

    // Webform mapping field to map webform element values to Person fields.
    $form['mapping']['user_field_mapping'] = [
      '#type' => 'webform_mapping',
      '#description' => $this->t('Map webform element values to Persons fields.'),
      '#required' => FALSE,
      '#source' => $mapping_options['source'],
      '#destination' => $mapping_options['destination'],
      '#default_value' => $this->configuration['user_field_mapping'],
      '#source__title' => $this->t('Persons Webform element'),
      '#destination__type' => 'select',
      '#destination__title' => $this->t('Persons field destination'),
      '#destination__description' => NULL,
      '#parents' => ['settings', 'user_field_mapping'],
    ];

    // Container for checkboxes related to Person Deals API.
    $form['mapping']['user_field_mapping']['checkbox_container'] = [
      '#type' => 'container',
      '#weight' => 10,
    ];

    // Checkbox to enable or disable Person Deals API.
    $form['mapping']['user_field_mapping']['checkbox_container']['enable_deals'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Person Deals API'),
      '#description' => $this->t('Check this box to enable Deals API for Person fields.'),
      '#default_value' => $this->configuration['enable_deals'],
      '#parents' => ['settings', 'enable_deals'],
      '#states' => [
        'visible' => [
          ':input[name="settings[enable_mapping]"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $default_tokens = [
      '[webform_submission:values]' => $this->t('All submission data'),
    ];

    // Combine all tokens into a string.
    $token_string = implode("\n", array_keys($default_tokens));
    // Prepare the form for tokenized text input.
    $form['mapping']['tokenized_text'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Notes'),
      '#description' => $this->t('Enter your text here, and you can use the available tokens.'),
      '#default_value' => $this->configuration['tokenized_text']['value'] ?? $token_string,
      '#format' => $this->configuration['tokenized_text']['format'] ?? 'full_html',
      '#parents' => ['settings', 'tokenized_text'],
      '#tokens' => $default_tokens,
    ];

    // Return the constructed form.
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    parent::validateConfigurationForm($form, $form_state);

    // Validate Person field mapping.
    if (empty($form_state->getValue('user_field_mapping'))) {
      $form_state->setErrorByName('user_field_mapping', $this->t('At least one Person mapping field must be selected.'));
    }

    // Validate organization mapping when enabled.
    if ($form_state->getValue('enable_mapping')) {
      $user_field_mapping_org = $form_state->getValue('user_field_mapping_org');
      if (empty($user_field_mapping_org)) {
        $form_state->setErrorByName(
          'user_field_mapping_org',
          $this->t('You must map at least one organisation field when "Is Organisation" is checked.')
        );
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    parent::submitConfigurationForm($form, $form_state);

    // You can access form values like this:
    $tokenized_text = $form_state->getValue(['settings', 'tokenized_text']);
    $user_field_mapping = $form_state->getValue(['settings', 'user_field_mapping']);

    // Optionally, save the custom configuration values to the handler.
    $this->configuration['tokenized_text'] = $tokenized_text;
    $this->configuration['user_field_mapping'] = $user_field_mapping;

    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE): void {
    // Fetch submission data and configuration settings.
    $webform_data = $webform_submission->getData();
    $user_field_mapping = $this->configuration['user_field_mapping'];
    $enableMapping = $this->configuration['enable_mapping'];
    $enable_deals = $this->configuration['enable_deals'];
    $enable_orgdeals = $this->configuration['enable_orgdeals'];

    // To get configuration for the pipedrive from service.
    $configsettings = \Drupal::service('webform_pipedrive.get_access_token')->accessToken();
    $endPoint = $configsettings['endpoint'];
    $accessToken = $configsettings['accessToken'];
    $orgId = 0;
    $user_field_mapping_org = [];

    // Initialize the formatted output.
    $formatted_output = '';
    // Format the submission values as key: value pairs with new lines.
    foreach ($webform_data as $key => $value) {
      $formatted_output .= ucfirst(str_replace('_', ' ', $key)) . ': ' . $value . "\n";
    }

    // Replace the token in the tokenized text.
    $token_service = \Drupal::token();
    $replaced_text = $token_service->replace($this->configuration['tokenized_text']['value'], ['webform_submission' => $webform_submission]);
    // Replace the tokenized values with the formatted output.
    $replaced_text = str_replace('[webform_submission:values]', $formatted_output, $replaced_text);
    // Optionally, render with HTML line breaks.
    $formatted_text = nl2br($replaced_text);

    // Process organization data.
    $orgData = [];
    if ($enableMapping) {
      $user_field_mapping_org = $this->configuration['user_field_mapping_org'];
      foreach ($user_field_mapping_org as $webform_key => $user_field) {
        $orgData[$user_field] = $webform_data[$webform_key];
      }
      $orgId = \Drupal::service('webform_pipedrive.pipe_drive_integration')->sentWebformSubmissionToPipeDriveForOrganisation($orgData, $accessToken, $endPoint);
    }

    // Process person data.
    $personData = [];
    $personData['org_id'] = $orgId;
    foreach ($user_field_mapping as $webform_key => $user_field) {
      $personData[$user_field] = $webform_data[$webform_key];
    }
    \Drupal::service('webform_pipedrive.pipe_drive_integration')->sentWebformSubmissionToPipeDriveForPersonWithDeal($personData, $accessToken, $endPoint, $orgId, $enable_deals, $enable_orgdeals, $formatted_text);
  }

  /**
   * Helper function to fetch mapping options.
   *
   * @return array
   *   Array of source and destination options.
   */
  protected function getMappingOptions(): array {
    // Get all Webform elements.
    $webform = $this->getWebform();
    $elements = $webform->getElementsInitializedFlattenedAndHasValue();
    $source_options = [];
    foreach ($elements as $key => $element) {
      $source_options[$key] = $element['#admin_title'] ?: $element['#title'] ?: $key;
    }

    // Fetch dynamic Pipedrive fields from the API.
    $destination_options = $this->fetchPipedriveFields();
    $orgdestination_options = $this->fetchOrgPipedriveFields();

    return [
      'source' => $source_options,
      'destination' => $destination_options,
      'orgdestination' => $orgdestination_options,
    ];
  }

  /**
   * Fetch the available Pipedrive fields from the API dynamically.
   *
   * @return array
   *   Array of Pipedrive fields.
   */
  protected function fetchPipedriveFields(): array {
    // To get configuration for the Pipedrive from the service.
    $configsettings = \Drupal::service('webform_pipedrive.get_access_token')->accessToken();
    $endPoint = $configsettings['endpoint'];
    $accessToken = $configsettings['accessToken'];

    $url = $endPoint . '/v1/personFields?api_token=' . $accessToken;

    try {
      $response = $this->httpClient->get($url);
      $response_body = $response->getBody()->getContents();
      $data = json_decode($response_body, TRUE);

      if (isset($data['data']) && is_array($data['data'])) {
        $destination_options = [];
        foreach ($data['data'] as $field) {
          $destination_options[$field['key']] = $field['name'];
        }
        return $destination_options;
      }
    }
    catch (RequestException $e) {
      $this->logger->error($this->t('Error Fetching Pipedrive Fields: @message', ['@message' => $e->getMessage()]));
    }

    // Return empty array if there is an error fetching fields.
    return [];
  }

  /**
   * Fetch the available Pipedrive organization fields from the API dynamically.
   *
   * @return array
   *   Array of Pipedrive organization fields.
   */
  protected function fetchOrgPipedriveFields(): array {
    // To get configuration for the Pipedrive from the service.
    $configsettings = \Drupal::service('webform_pipedrive.get_access_token')->accessToken();
    $endPoint = $configsettings['endpoint'];
    $accessToken = $configsettings['accessToken'];

    $url = $endPoint . '/v1/organizationFields?api_token=' . $accessToken;

    try {
      $response = $this->httpClient->get($url);
      $response_body = $response->getBody()->getContents();
      $data = json_decode($response_body, TRUE);

      if (isset($data['data']) && is_array($data['data'])) {
        $destination_options = [];
        foreach ($data['data'] as $field) {
          $destination_options[$field['key']] = $field['name'];
        }
        return $destination_options;
      }
    }
    catch (RequestException $e) {
      $this->logger->error($this->t('Error Fetching Pipedrive Fields: @message', ['@message' => $e->getMessage()]));
    }

    // Return empty array if there is an error fetching fields.
    return [];
  }

}
