<?php

namespace Drupal\webform_pipedrive\Service;

use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Psr\Log\LoggerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
/**
 * Service class for integrating with Pipedrive.
 */
class PipedriveIntegration {
  use StringTranslationTrait;
  /**
   * HTTP client service.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs the PipedriveIntegration service.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(ClientInterface $http_client, LoggerInterface $logger) {
    $this->httpClient = $http_client;
    $this->logger = $logger;
  }

  /**
   * Sends a webform submission to Pipedrive for creating an organization.
   *
   * @param array $orgData
   *   The organization data.
   * @param string $accessToken
   *   The access token.
   * @param string $endPoint
   *   The API endpoint.
   *
   * @return string
   *   The created organization ID.
   */
  public function sentWebformSubmissionToPipeDriveForOrganisation(array $orgData, $accessToken, $endPoint) {
    $org_id = '';
    $endpoint_api_url = $endPoint . "/v1/organizations?api_token=" . $accessToken;

    try {
      // Make a POST request using the injected HTTP client.
      $response = $this->httpClient->request('POST', $endpoint_api_url, [
        'json' => $orgData,
        'headers' => [
          'Content-Type' => 'application/json',
        ],
      ]);

      // Process the response body.
      $response_body = $response->getBody()->getContents();
      $data = json_decode($response_body, TRUE);
      if ($data && isset($data['data']['id'])) {
        $org_id = $data['data']['id'];
      }
    }
    catch (RequestException $e) {
      // Log any errors.
      $this->logger->error($this->t('Error creating organization: @message', ['@message' => $e->getMessage()]));
    }

    return $org_id;
  }

  /**
   * Sends a webform submission to Pipedrive for a person with an optional deal.
   *
   * @param array $personData
   *   The person data.
   * @param string $accessToken
   *   The access token.
   * @param string $endPoint
   *   The API endpoint.
   * @param int $orgId
   *   The organization ID.
   * @param bool $enable_deals
   *   Whether to enable deals.
   * @param bool $enable_orgdeals
   *   Whether to enable organization-specific deals.
   * @param string $formatted_text
   *   The tokenized content to be sent as a note associated with the person.
   */
  public function sentWebformSubmissionToPipeDriveForPersonWithDeal(array $personData, $accessToken, $endPoint, $orgId, $enable_deals, $enable_orgdeals, $formatted_text) {
    $endpoint_api_url = $endPoint . "/v1/persons?api_token=" . $accessToken;

    try {
      // Make a POST request to create a person.
      $response = $this->httpClient->request('POST', $endpoint_api_url, [
        'json' => $personData,
        'headers' => [
          'Content-Type' => 'application/json',
        ],
      ]);

      // Process the response body for the person.
      $response_body = $response->getBody()->getContents();
      $data = json_decode($response_body, TRUE);
      $person_id = $data['data']['id'] ?? 0;

      // Create a deal if needed.
      if (($enable_deals && !$orgId) || ($enable_orgdeals && $orgId)) {
        $deals = [
          'title' => '[Deal] ' . $personData['name'],
          'person_id' => $person_id,
          'org_id' => $orgId,
        ];
        $dealUrl = $endPoint . '/v1/deals?api_token=' . $accessToken;
        $dealResponse = $this->httpClient->request('POST', $dealUrl, [
          'json' => $deals,
          'headers' => [
            'Content-Type' => 'application/json',
          ],
        ]);

        // Process the response for the deal.
        $dealResponseBody = $dealResponse->getBody()->getContents();
        $dealData = json_decode($dealResponseBody, TRUE);
        // Log the deal creation if needed.
        if (isset($dealData['data'])) {
          $this->logger->info($this->t('Deal created with ID: @deal_id', ['@deal_id' => $dealData['data']['id']]));
        }
        else {
          $this->logger->warning($this->t('Failed to create deal: @response', ['@response' => $dealResponseBody]));
        }
      }
      // Check if person was created successfully.
      if ($person_id) {
        // Now send the replaced token text as a note.
        $noteData = [
          'content' => $formatted_text,
          'person_id' => $person_id,
        ];

        // API URL for creating a note in Pipedrive.
        $note_url = $endPoint . '/v1/notes?api_token=' . $accessToken;

        // Make a POST request to create a note.
        $noteResponse = $this->httpClient->request('POST', $note_url, [
          'json' => $noteData,
          'headers' => [
            'Content-Type' => 'application/json',
          ],
        ]);

        // Process the note creation response.
        $noteResponseBody = $noteResponse->getBody()->getContents();
        $noteData = json_decode($noteResponseBody, TRUE);
        if (isset($noteData['data'])) {
          $this->logger->info($this->t('Note created with ID: @note_id', ['@note_id' => $noteData['data']['id']]));
        }
        else {
          $this->logger->warning($this->t('Failed to create note: @response', ['@response' => $noteResponseBody]));
        }
      }
    }
    catch (RequestException $e) {
      // Log any errors.
      $this->logger->error($this->t('Error creating person or deal: @message', ['@message' => $e->getMessage()]));
    }
  }

}
